//  *****************************************************************************
//                                    main.c
// 
//        Demonstracny program pre dosku KTK ARM7
//
//        Komunikacia s PC cez USB - virtualny COM port
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"
#include "usb_com.h"

// Globalne premenne
struct _AT91S_CDC pCDC;           // struktura pre USB komunikaciu


//  *******************************************************
//  Cakanie cca 450 us
//  *******************************************************
void cakaj(void)
{ unsigned int cas;
 
  for(cas=0; cas < MCK/20000; cas++) ;
}

//  *******************************************************
//  Otvorenie USB portu
//  *******************************************************
void AT91F_USB_Open(void)
{
  // Nastavenie PLL USB Divider-a
  AT91C_BASE_CKGR->CKGR_PLLR |= AT91C_CKGR_USBDIV_1 ;

  // Povolenie hodin - 48 MHz a periferne hodiny
  AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_UDP;
  AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_UDP);

  // inicializacia CDC struktury
  AT91F_CDC_Open(&pCDC, AT91C_BASE_UDP);

  // pripojenie pull-up odporu
  AT91C_BASE_PIOA->PIO_SODR = USBEN;
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main (void) 
{ char data[64], text[20];
  unsigned int pocet;

  //******************* Uvodna inicializacia *******************

  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);

  //******************* Inicializacia portu A ******************
  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
  // konfiguracia jednotlivych pinov
  // vstupne piny
  AT91C_BASE_PIOA->PIO_ODR = S2 | USBPWR;
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = D1 | USBEN;
  // povolenie Pull-Up rezistoru
  AT91C_BASE_PIOA->PIO_PPUER = S2;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = D1 | S2 | USBEN | USBPWR;
  // vsetko je vypnute
  AT91C_BASE_PIOA->PIO_CODR = D1 | USBEN;

  //******************* Hlavna cast programu *******************

  // priprava retazca na vyslanie
  text[0]='U';
  text[1]='S';
  text[2]='B';
  text[3]=' ';
  text[4]='P';
  text[5]='A';
  text[6]='K';
  text[7]='E';
  text[8]='T';
  text[9]=' ';
  text[10]='0';
  text[11]=13;       // novy riadok - CR, LF
  text[12]=10;
  text[13]=0;

  // zapnutie LED - presla uvodna faza inicializacie
  AT91C_BASE_PIOA->PIO_SODR = D1;

  // cakanie na pripojenie USB
  while(!(AT91C_BASE_PIOA->PIO_PDSR & USBPWR));

  // start inicializacie USB - zhasne LED
  AT91C_BASE_PIOA->PIO_CODR = D1;
  
  // inicializacia USB komunikacie
  AT91F_USB_Open();

  // cakanie na ukoncenie enumeracie
  while (!pCDC.IsConfigured(&pCDC));

  // USB inicializacia ukoncena - zapnutie LED
  AT91C_BASE_PIOA->PIO_SODR = D1;

  // cakanie na 'S' paket, alebo stlacenie S2
  while(1)
  { // ak bolo stlacene tlacidlo, vysle sa retazec cez USB
    if (!(AT91C_BASE_PIOA->PIO_PDSR & S2))
    { // uprava retazca - zmena cisla paketu od 0 - 9
      if (text[10] == '9')
        text[10] = '0';
       else
        text[10]++;

       // vyslanie paketu
      pCDC.Write(&pCDC, text, 13);

      // cakanie na pustenie tlacidla
      while(!(AT91C_BASE_PIOA->PIO_PDSR & S2));
        cakaj();
    }

    // kontrola, ci nie su k dispozicii nejake data z USB
    if (pCDC.IsData(&pCDC))
    { // nacitanie prijatych dat
      pocet = pCDC.Read(&pCDC, data, 64);

      // znaky sa prijimaju po jednom
      if (pocet != 1)
        continue;

      // kontrola, ci bol prijaty prikaz 'S'
      if (data[0] == 'S' || data[0] == 's')
      { // uprava retazca - zmena cisla paketu od 0 - 9
        if (text[10] == '9')
          text[10] = '0';
         else
          text[10]++;

        // vyslanie paketu
        pCDC.Write(&pCDC, text, 13);
      }  
    }
  }
}
